#ifndef MBOCC_H
#define MBOCC_H

#include <iostream>
#include <locale>
#include <codecvt>
#include <chrono>

#include <TDataStd_AsciiString.hxx>
#include <TDataStd_BooleanArray.hxx>
#include <TDataStd_BooleanList.hxx>
#include <TDataStd_ByteArray.hxx>
#include <TDataStd_Comment.hxx>
#include <TDataStd_ExtStringArray.hxx>
#include <TDataStd_ExtStringList.hxx>
#include <TDataStd_Integer.hxx>
#include <TDataStd_IntegerArray.hxx>
#include <TDataStd_IntegerList.hxx>
#include <TDataStd_Name.hxx>
#include <TDataStd_Real.hxx>
#include <TDataStd_RealArray.hxx>
#include <TDataStd_RealList.hxx>
#include <TDataStd_ReferenceArray.hxx>
#include <TDataStd_ReferenceList.hxx>
#include <TDataStd_Relation.hxx>
#include <TDataStd_UAttribute.hxx>
#include <TDataStd_Variable.hxx>
#include <TDF_AttributeIterator.hxx>
#include <TDF_ChildIterator.hxx>
#include <TDF_ChildIDIterator.hxx>
#include <TDF_Reference.hxx>
#include <TDF_TagSource.hxx>
#include <TDF_Tool.hxx>
#include <TNaming_NamedShape.hxx>
#include <TNaming_Iterator.hxx>
#include <BRepTools.hxx>
#include <TopoDS_Shape.hxx>
#include <TopExp_Explorer.hxx>



//--------------------------------------------------------------------------
// Function declarations
//--------------------------------------------------------------------------
static std::string ShapeTypeStr(const TopoDS_Shape shp);
static std::string EvolutionStr(const TNaming_Evolution evol);

static void DumpEvolution(Handle(TNaming_NamedShape) aNS);
static void DumpShapeHistory(const TDF_Label& theLabel, const TopoDS_Shape& theShape);

static void DumpVar(const char *szName, Handle(TNaming_NamedShape) aNS);
static void DumpVar(const char *szName, Handle(TDF_Attribute) aAttr);
static void DumpVar(const char *szName, const TDF_Label &lbl);
static void DumpVar(const char *szName, const TopoDS_Shape &aShape);





//==========================================================================
static std::string ShapeTypeStr(const TopoDS_Shape shp) {
	if (shp.IsNull())
		return "null";

	switch (shp.ShapeType()) {
		case TopAbs_COMPOUND: 	return "COMPOUND";
		case TopAbs_COMPSOLID:	return "COMPSOLID";
		case TopAbs_SOLID: 		return "SOLID";
		case TopAbs_SHELL: 		return "SHELL";
		case TopAbs_FACE: 		return "FACE";
		case TopAbs_WIRE: 		return "WIRE";
		case TopAbs_EDGE: 		return "EDGE";
		case TopAbs_VERTEX: 	return "VERTEX";
		case TopAbs_SHAPE: 		return "SHAPE";
	}
	return "??";
}

//==========================================================================
static std::string EvolutionStr(const TNaming_Evolution evol) {
	switch (evol) {
		case TNaming_PRIMITIVE:	return "PRIMITIVE";
		case TNaming_GENERATED:	return "GENERATED";
		case TNaming_MODIFY:	return "MODIFY";
		case TNaming_DELETE:	return "DELETE";
		case TNaming_REPLACE:	return "REPLACE";
		case TNaming_SELECTED:	return "SELECTED";
	}
	return "??";
}

//==========================================================================
static void SaveShape(const TopoDS_Shape &shp, const char* szFileName) {
	std::string fn("/home/bernhard/brep/");
	std::chrono::high_resolution_clock::time_point now_t = std::chrono::high_resolution_clock::now();
	unsigned long long now_c = *((unsigned long long*)&now_t);
	fn = fn + std::to_string(now_c) + "_" + szFileName + ".brep";
	MSGEL("...saving shape \"" << szFileName << "\"to [" << fn << "]");
	BRepTools::Write(shp, fn.c_str());
}

//==========================================================================
static void DumpChildren(const TDF_Label& aLabel) {
	TDF_ChildIterator it;
	//TCollection_AsciiString as;
	for	(it.Initialize(aLabel); it.More(); it.Next()) {
		SHOW(it.Value());
		DumpChildren(it.Value());
		// TDF_Tool::Entry(it.Value(),as);
		// MSGEL("* [Label]: child=" << as.ToCString());
	}
 }

//==========================================================================
static void DumpShapeHistory(const TDF_Label& theLabel, const TopoDS_Shape& theShape)
{
  SHOW(theLabel);
  SHOW(theShape);
  DLOG << "{-------ShapeHistory-------------------------" << std::endl;
  TopoDS_Shape anOldShape; // old shape in the pair old shape->theShape in the named shape
  TopoDS_Shape aShapeContainer; // old shape of the shape that contains theShape as sub-element
  Handle(TNaming_NamedShape) aCandidatInThis, aCandidatContainer;
  // use child and this label (the lowest priority)
  TDF_ChildIDIterator aNSIter(theLabel, TNaming_NamedShape::GetID(), Standard_True);
  bool aUseThis = !aNSIter.More();
  while(anOldShape.IsNull() && (aNSIter.More() || aUseThis)) {
    Handle(TNaming_NamedShape) aNS;
    if (aUseThis) {
      if (!theLabel.FindAttribute(TNaming_NamedShape::GetID(), aNS))
        break;
    } else {
      aNS = Handle(TNaming_NamedShape)::DownCast(aNSIter.Value());
    }
    for(TNaming_Iterator aShapesIter(aNS); aShapesIter.More(); aShapesIter.Next()) {
      if (aShapesIter.Evolution() == TNaming_SELECTED || aShapesIter.Evolution() == TNaming_DELETE)
        continue; // don't use the selection evolution

      if (aShapesIter.NewShape().IsSame(theShape)) { // found the original shape
        aCandidatInThis = aNS;
        if (aCandidatInThis->Evolution() == TNaming_MODIFY) {
          anOldShape = aShapesIter.OldShape();
          MSG("* modified from: "); SHOW(anOldShape);
        }
        // otherwise may we searching for another item of this shape with longer history
        if (!anOldShape.IsNull())
          break;
      }
      // check that the shape contains theShape as sub-shape to fill container
      if (aShapesIter.NewShape().ShapeType() < theShape.ShapeType() && aCandidatContainer.IsNull()) {
        TopExp_Explorer anExp(aShapesIter.NewShape(), theShape.ShapeType());
        for(; anExp.More(); anExp.Next()) {
          if (theShape.IsSame(anExp.Current())) {
            aCandidatContainer = aNS;
            aShapeContainer = aShapesIter.NewShape();
          }
        }
      }
    }
    // iterate to the next label or to the body label in the end
    if (!aUseThis)
      aNSIter.Next();
    if (!aNSIter.More()) {
      if (aUseThis)
        break;
      aUseThis = true;
    }
  }
  DLOG << "}--------------------------------------------" << std::endl;
}



//==========================================================================
static void DumpEvolution(Handle(TNaming_NamedShape) aNS) {
  DLOG << "{";
  // TNaming_Iterator iterates in reversed way than it was put in Builder, so, keep it in container
  // and iterate from end to begin
  NCollection_List<TopoDS_Shape> aOlds;
  NCollection_List<TopoDS_Shape> aNews;
  NCollection_List<TNaming_Evolution> aEvol;
  for (TNaming_Iterator anIt(aNS); anIt.More(); anIt.Next()) {
    aOlds.Prepend(anIt.OldShape());
    aNews.Prepend(anIt.NewShape());
    aEvol.Prepend(anIt.Evolution());
  }
  NCollection_List<TopoDS_Shape>::Iterator aOldIter(aOlds);
  NCollection_List<TopoDS_Shape>::Iterator aNewIter(aNews);
  NCollection_List<TNaming_Evolution>::Iterator aEvolIter(aEvol);
  for(; aOldIter.More(); aOldIter.Next(), aNewIter.Next(), aEvolIter.Next()) {
    switch (aEvolIter.Value()) {
    case TNaming_PRIMITIVE :
      DLOG << "  * primitive " << ShapeTypeStr(aNewIter.Value()) << std::endl;
      break;
    case TNaming_GENERATED :
      DLOG << "  * generated from " << ShapeTypeStr(aOldIter.Value()) << " to " << ShapeTypeStr(aNewIter.Value()) << std::endl;
      break;
    case TNaming_MODIFY :
      DLOG << "  * modified from " << ShapeTypeStr(aOldIter.Value()) << " to " << ShapeTypeStr(aNewIter.Value()) << std::endl;
      break;
    case TNaming_DELETE :
      DLOG << "  * deleted " << ShapeTypeStr(aOldIter.Value()) << std::endl;
      break;
    case TNaming_SELECTED :
      DLOG << "  * selected in " << ShapeTypeStr(aNewIter.Value()) << " a(n) " << ShapeTypeStr(aOldIter.Value()) << std::endl;
      break;
    default:
      break;
    }
  }
  DLOG << "}" << std::endl;
}

//==========================================================================
static void DumpVar(const char *szName, Handle(TNaming_NamedShape) aNS) {
	if (aNS.IsNull()) {
		DLOG << "[NamShp]: " << szName << "=Null" << std::endl;
		return;
	}
	DLOG << "[NamShp]: " << szName << " vers=" << aNS->Version() << " Evol=[" << EvolutionStr(aNS->Evolution()) << "] Shape=\"" << ShapeTypeStr(aNS->Get()) << "\"" << std::endl;
  //DumpEvolution(aNS);
}

//==========================================================================
static void DumpVar(const char *szName, Handle(TDF_Attribute) attr) {
	if (attr.IsNull()) {
		DLOG << "[TDFAttr]: " << szName << "=Null" << std::endl;
		return;
	}
	int trans = attr->Transaction();
	std::string state(" state=[");
	state = state + (attr->IsValid() ? "valid" : "invalid");
	state = state + "|" + (attr->IsNew() ? "new" : "old");
	state = state + "|" + (attr->IsForgotten() ? "forgot" : "known") + "]";
	Handle(TDataStd_AsciiString) aAS = Handle(TDataStd_AsciiString)::DownCast(attr);
	if (!aAS.IsNull()) {
		DLOG << "[TDFAttr_AscStr]: " << szName << "=\"" << aAS->Get() << "\"" << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDataStd_BooleanArray) aBoolArr = Handle(TDataStd_BooleanArray)::DownCast(attr);
	if (!aBoolArr.IsNull()) {
		DLOG << "[TDFAttr_BoolArr]: " << szName << "=" << aBoolArr->Length() << " [";
		if (aBoolArr->Length() > 0) {
			for (int i=aBoolArr->Lower(); i<=aBoolArr->Upper(); i++) {
				DLOG << (i == aBoolArr->Lower() ? "" : ",") << YesNo(aBoolArr->Value(i));
			}
		}
		DLOG << "]" << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDataStd_BooleanList) aBoolLst = Handle(TDataStd_BooleanList)::DownCast(attr);
	if (!aBoolLst.IsNull()) {
		const TDataStd_ListOfByte& lst = aBoolLst->List();
		DLOG << "[TDFAttr_BoolLst]: " << szName << "=" << lst.Size() << " [";
		if (!lst.IsEmpty()) {
			for (auto b=lst.cbegin(); b!=lst.cend(); ++b) {
				DLOG << (b == lst.cbegin() ? "" : ",") << YesNo(*b);
			}
		}
		DLOG << "]" << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDataStd_Comment) aCmt = Handle(TDataStd_Comment)::DownCast(attr);
	if (!aCmt.IsNull()) {
		DLOG << "[TDFAttr_Comment]: " << szName << "=\"" << aCmt->Get() << "\"" << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDataStd_ExtStringList) aESLst = Handle(TDataStd_ExtStringList)::DownCast(attr);
	if (!aESLst.IsNull()) {
		const TDataStd_ListOfExtendedString& lst = aESLst->List();
		DLOG << "[TDFAttr_ExStrLst]: " << szName << "=" << lst.Size() << " [";
		if (!lst.IsEmpty()) {
			for (auto str=lst.cbegin(); str!=lst.cend(); ++str) {
				//std::wstring ws((wchar_t*)(*str).ToExtString());
				DLOG << (str == lst.cbegin() ? "" : ",") << "\"" << *str << "\"";
			}
		}
		DLOG << "]" << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDataStd_Integer) aInt = Handle(TDataStd_Integer)::DownCast(attr);
	if (!aInt.IsNull()) {
		DLOG << "[TDFAttr_Int]: " << szName << "=" << aInt->Get() << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDataStd_IntegerArray) aIntArr = Handle(TDataStd_IntegerArray)::DownCast(attr);
	if (!aIntArr.IsNull()) {
		DLOG << "[TDFAttr_IntArr]: " << szName << "=" << aIntArr->Length() << " [";
		if (aIntArr->Length() > 0) {
			for (int i=aIntArr->Lower(); i<=aIntArr->Upper(); i++) {
				DLOG << (i == aIntArr->Lower() ? "" : ",") << aIntArr->Value(i);
			}
		}
		DLOG << "]" << " trans=" << trans << state << std::endl;
		return;
	}

	Handle(TDataStd_Name) aNam = Handle(TDataStd_Name)::DownCast(attr);
	if (!aNam.IsNull()) {
		DLOG << "[TDFAttr_Name]: " << szName << "=\"" << aNam->Get() << "\"" << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TNaming_NamedShape) aNS = Handle(TNaming_NamedShape)::DownCast(attr);
	if (!aNS.IsNull()) { 
		DLOG << "[TDFAttr_NamShp]: " << szName << "=\"" << ShapeTypeStr(aNS->Get()) << "\" trans=" << trans << state << std::endl;
		DLOG << aNS;
		TNaming_Iterator it(aNS);
		for (; it.More(); it.Next()) {
			MSGEL("[NamShp]: child: Modif=" << YesNo(it.IsModification()) << " Evol=[" << EvolutionStr(it.Evolution()) << "] Old=\"" << ShapeTypeStr(it.OldShape()) << "\" New=\""<< ShapeTypeStr(it.NewShape()) << "\"");
		}
    //SHOW(aNS);
		return;
	}
	Handle(TDataStd_Real) aReal = Handle(TDataStd_Real)::DownCast(attr);
	if (!aReal.IsNull()) {
		DLOG << "[TDFAttr_Real]: " << szName << "=" << aReal->Get() << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDF_Reference) aRef = Handle(TDF_Reference)::DownCast(attr);
	if (!aRef.IsNull()) {
		TCollection_AsciiString entry;
		TDF_Tool::Entry(aRef->Get(), entry);
		DLOG << "[TDFAttr_Ref]: " << szName << "=" << entry << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDataStd_ReferenceArray) aRefArr = Handle(TDataStd_ReferenceArray)::DownCast(attr);
	if (!aRefArr.IsNull()) {
		DLOG << "[TDFAttr_RefArr]: " << szName << "=" << aRefArr->Length() << " [";
		if (aRefArr->Length() > 0) {
			TCollection_AsciiString entry;
			for (int i=aRefArr->Lower(); i<=aRefArr->Upper(); i++) {
				TDF_Tool::Entry(aRefArr->Value(i), entry);
				DLOG << (i == aRefArr->Lower() ? "" : ",") << entry;
			}
		}
		DLOG << "]" << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDataStd_ReferenceList) aRefLst = Handle(TDataStd_ReferenceList)::DownCast(attr);
	if (!aRefLst.IsNull()) {
		const TDF_LabelList& lst = aRefLst->List();
		DLOG << "[TDFAttr_RefLst]: " << szName << "=" << lst.Size() << " [";
		if (!lst.IsEmpty()) {
			TCollection_AsciiString entry;
			for (auto ref=lst.cbegin(); ref!=lst.cend(); ++ref) {
				TDF_Tool::Entry(*ref, entry);
				DLOG << (ref == lst.cbegin() ? "" : ",") << entry;
			}
		}
		DLOG << "]" << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDF_TagSource) aTS = Handle(TDF_TagSource)::DownCast(attr);
	if (!aTS.IsNull()) {
		DLOG << "[TDFAttr_TagSrc]: " << szName << "=" << aTS->Get() << " trans=" << trans << state << std::endl;
		return;
	}
	Handle(TDataStd_UAttribute) aUA = Handle(TDataStd_UAttribute)::DownCast(attr);
	if (!aUA.IsNull()) {
		DLOG << "[TDFAttr_UAttr]: " << szName << "= trans=" << trans << state << std::endl;
		return;
	}
  DLOG << "[" << attr->DynamicType()->Name() << "]" << attr << std::endl;
}

//==========================================================================
static void DumpVar(const char *szName, const TDF_Label &lbl) {
	try {
		if (lbl.IsNull()) {
			DLOG << "[Label]: " << szName << "=Null" << std::endl;
		} else {
			TCollection_AsciiString entry; TDF_Tool::Entry(lbl, entry);
			std::string imp = (lbl.IsImported() ? "import" : "noimp");
			std::string modif = (lbl.MayBeModified() ? "modif" : "nomodif");
			int nAttr = (lbl.HasAttribute() ? lbl.NbAttributes() : 0);
			int nChild = (lbl.HasChild() ? lbl.NbChildren() : 0);
      DLOG << "[Label]: " << szName << "={" << entry << " state=[" << imp << "|" << modif << "] nAttr=" << nAttr << " nChild=" << nChild << (nChild > 0 || nAttr > 0 ? "" : "}") << std::endl;
			if (nAttr > 0) {
				// Dump all attributes
				for (TDF_AttributeIterator itr(lbl); itr.More(); itr.Next()) {
					Handle(TDF_Attribute) attr = itr.Value();
					SHOW(attr);
				}
			}
			if (nChild > 0) {
				TDF_Label child;
				for	(TDF_ChildIterator it(lbl); it.More(); it.Next()) {
					child = it.Value();
					SHOW(child);
				}
			}
			if (nAttr > 0 || nChild > 0) {
				MSGEL("}");
			}
		}
	}
	catch (...) { }
}

//==========================================================================
static void DumpVar(const char *szName, const TopoDS_Shape &aShape) {
	if (aShape.IsNull()) {
		DLOG << "[TopShp]: " << szName << "=Null" << std::endl;
	} else {
		std::string typ = ShapeTypeStr(aShape);
		DLOG << "[TopShp]: " << szName << "=" << aShape.TShape().get() << " ShapeType=" << typ;
		if (aShape.ShapeType() == TopAbs_COMPOUND) {
      int children[TopAbs_SHAPE+1] = {0};
      int nbChild = 0;
      for (TopoDS_Iterator Iter(aShape); Iter.More(); Iter.Next()) {
        if (!Iter.Value().IsNull()) {
          children[(int)Iter.Value().ShapeType()]++;
          nbChild++;
        }
      }
			DLOG << " NumChild=" << nbChild;
      if (nbChild > 0) {
        bool bFirst = true;
        DLOG << " (";
        for (int i=TopAbs_COMPOUND; i<TopAbs_SHAPE; i++) {
          if (children[i] > 0) {
            if (!bFirst) DLOG << ", ";
            DLOG << children[i] << " " << TopAbs::ShapeTypeToString((TopAbs_ShapeEnum)i);
            bFirst = false;
          }
        }
        DLOG << ")";
      }
		}
    else if (aShape.ShapeType() == TopAbs_COMPSOLID) {
			DLOG << " NumChild=" << aShape.NbChildren();
		}
		DLOG << std::endl;
	}
}

#endif // MBOCC_H